info = {
    'name': 'arb',
    'version': '1.0.0',
    'description': 'Application Resource Bundle (ARB)',
    'file-mode': "language-file",
    'file-extensions': [ 'arb' ],
    'delimiter': '_',
    'modifies-existing-files': true,
    'primary-language-is-mandatory': true,
    'requires-reload-on-primary-language-change': true,
    'enable-parameter-editor': true,
    'copy-template-defaults': [ 'Localizations.of(context).%1' ],

    configuration: [
        common.configuration[0], // indent
        {
            'name': 'format',
            'displayName': 'Format',
            'type': 'enum',
            'default': 'plaintext-json',
            'values': {
                'plaintext-json': 'ARB JSON format',
                'json': 'ARB JSON with "_" as namespace delimiter'
            }
        }
    ],

    'formats': common.formats
};


fileio = {

    //
    // Imports file contents
    //
    // filecontent: content of the file as string
    //
    importData: function(filecontent, fileformat)
    {
        if(babeledit.checkAndReportJsonError(filecontent, 0))
        {
            return;
        }

        var data = JSON.parse(filecontent);

        for (var key in data)
        {
            if (typeof data[key] === "string" && !key.startsWith("@"))
            {
                var treePath = [ key ];
                if (fileformat && info.formats[fileformat].flags['split-join-keys'])
                {
                    treePath = key.split(info.delimiter);
                }

                babeledit.addTranslation(treePath, data[key]);

                const metaKey = "@" + key;
                var comment = "";
                var params = {};
                if (metaKey in data && "description" in data[metaKey])
                {
                    comment = data[metaKey].description;
                }
                if (metaKey in data && "placeholders" in data[metaKey])
                {
                    params = data[metaKey].placeholders;
                }
                babeledit.setDescription(treePath, "");
                babeledit.setComment(treePath, comment);
                babeledit.setParameters(treePath, params, Object.keys(params));
            }
        }
    },

    //
    // prepare writing of files
    //
    prepareExport: function(filename, configuration)
    {
        common.prepareExport(filename, configuration);
    },


    //
    // save translation for current file
    //
    saveTranslation: function(fullPath, translation)
    {
        if (fullPath.length != 1)
        {
            fullPath = [ fullPath.join(info.delimiter) ];
        }
        common.documentRoot[fullPath[0]] = translation;
    },


    //
    // save meta data for translation
    //
    addMetadata: function(fullPath, comment, params)
    {
        if (fullPath.length != 1)
        {
            fullPath = [ fullPath.join(info.delimiter) ];
        }
        var key = "@" + fullPath[0];
        common.documentRoot[key] = { };
        common.documentRoot[key].description = comment;
        common.documentRoot[key].placeholders = params;
    },


    //
    // return formatted file data
    //
    getSaveFile: function(filecontent)
    {
        var oldData = {};
        if (filecontent.length)
        {
            try
            {
                oldData = JSON.parse(filecontent);
            }
            catch (e)
            {
                throw Error("Failed to parse old file content (" + e.message + ")");
            }
        }

        var newData = common.documentRoot;
        //console.log(JSON.stringify(newData));

        // remove translations + metadata which no longer exist from JSON file
        for (var key in oldData)
        {
            if (!key.startsWith("@") && !(key in newData))
            {
                delete oldData[key];
                delete oldData['@' + key];
            }
        }

        // insert translations into existing JSON file
        for (key in newData)
        {
            if (!key.startsWith("@"))
            {
                // set translation text
                oldData[key] = newData[key];

                // merge metadata block (description+placeholders are set by BE, other properties are kept)
                const meta = "@" + key;
                if (newData[meta])
                {
                    oldData[meta] = oldData[meta] ? Object.assign(oldData[meta], newData[meta])
                                                  : newData[meta];
                    // console.log("merged:", oldData[meta]);

                    // delete empty strings / objects
                    if (oldData[meta].description.length == 0)
                    {
                        delete oldData[meta].description
                    }
                    if (Object.keys(oldData[meta].placeholders).length == 0)
                    {
                        delete oldData[meta].placeholders
                    }
                    if (Object.keys(oldData[meta]).length == 0)
                    {
                        delete oldData[meta];
                    }
                }
            }
        }

        // fix sort order (text followed by meta data)
        var output = {};
        for (key in oldData)
        {
            if (key.startsWith("@@"))
            {
                output[key] = oldData[key];
                continue;
            }
            if (key.startsWith("@"))
            {
                continue;
            }
            output[key] = oldData[key];
            const meta = "@" + key;
            if (oldData[meta])
            {
                output[meta] = oldData[meta];
            }
        }

        common.documentRoot = output;
        return common.getSaveFile();
    }

}
