let rx="("+[

    '<FormattedMessage.*?</FormattedMessage>',

    '<FormattedHTMLMessage.*?</FormattedHTMLMessage>',

    // <FormattedHTMLMessage id="" values={{ what: 'react-intl' }} message=''/>
    String.raw`<(FormattedMessage|FormattedHTMLMessage\s*\S+\s*=\s*(%1|\{.*?\}))*?\s*/>`,

    '%1'

].join('|')+")";


info = {
    'name': 'react-formatjs-extract',
    'version': '1.0.0',
    'description': 'React with formatjs extract/compile',
    'file-mode': "language-file",
    'delimiter': '.',
    'file-extensions': [ 'json' ],
    'copy-template-defaults': [ '', '', '' ],
    'exclude-dirs': ['node_modules', 'vendor', 'dist'],
    'sources-to-scan': ['*.js', '*.jsx', '*.ts', '*.tsx'],
    'sources-to-exclude': [],
    'source-scanner-rx': rx,
    'primary-language-is-mandatory': true,
    'primary-language-is-readonly': true,
    'requires-reload-on-primary-language-change': true,


    'configuration': [
        {
            'name': 'indent',
            'displayName': 'Indent',
            'type': 'enum',
            'default': 'space2',
            'values': {
                'none': 'none',
                'space2': '2 spaces',
                'space4': '4 spaces',
                'tab': 'tab'
            }
        },
        {
            'name': 'format',
            'displayName': 'Format',
            'type': 'enum',
            'default': 'simple-tree',
            'values': {
                'default-tree': 'FormatJS default (hierarchical IDs, tree view)',
                'default-list': 'FormatJS default (flat IDs, list view)',
                'simple-tree': 'FormatJS simple (hierarchical IDs, tree view)',
                'simple-list': 'FormatJS simple (flat IDs, list view)'
            }
        },

    ],

    'formats': {
        'default-tree': {
            'flags': {
                'nested-keys': true,
                'split-join-keys': true,
                'add-remove-rename-concepts': false,
                'display-text-in-treeview': true
            },
            'list-fallback-format': 'default-list'
        },
        'default-list': {
            'flags': {
                'nested-keys': false,
                'split-join-keys': false,
                'add-remove-rename-concepts': false,
                'display-text-in-treeview': true
            }
        },
        'simple-tree': {
            'flags': {
                'nested-keys': true,
                'split-join-keys': true,
                'add-remove-rename-concepts': false,
                'display-text-in-treeview': true
            },
            'list-fallback-format': 'simple-list'
        },
        'simple-list': {
            'flags': {
                'nested-keys': false,
                'split-join-keys': false,
                'add-remove-rename-concepts': false,
                'display-text-in-treeview': true
            }
        },
    }

};

common.formats = info.formats;

fileio = {

    //
    // Imports file contents
    //
    // filecontent: content of the file as string
    //
    importData: function(filecontent, fileformat)
    {
        if (babeledit.checkAndReportJsonError(filecontent, 0))
        {
            return;
        }

        var data = JSON.parse(filecontent);

        if (!fileformat)
        {
            fileformat = 'simple-tree';
            babeledit.setFileFormat(fileformat);
        }


        for (var key in data)
        {
            var id, text, description;

            if (info.formats[fileformat].flags['split-join-keys'])
            {
                id = key.split(info.delimiter);
            }
            else
            {
                id = [key];
            }

            if (typeof data[key] === "object")
            {
                text = data[key].defaultMessage;
                description = data[key].description;
                babeledit.proposeAsPrimaryLanguage();
            }
            else
            {
                text = data[key];
                description = null;
            }

            if (typeof text === "string" && text.length > 0)
            {
                babeledit.addTranslation(id, text);
                babeledit.setDescription(id, description);
            }
        }
    },


    //
    // prepare writing of files
    //
    prepareExport: common.prepareExport,


    //
    // insert translation in document model
    //
    saveTranslation: function(fullPath, translation)
    {
        var fullName = fullPath.join(info.delimiter);
        if (common.config.format.startsWith('default-'))
        {
            common.documentRoot[fullName] = { 'defaultMessage': translation };
        }
        else
        {
            common.documentRoot[fullName] = translation;
        }
    },


    //
    // return formatted file data
    //
    getSaveFile: common.getSaveFile

}
